#this test is much more to test that **it's there** and answers attribute access
import autopath
from py.test import raises

from config import *
from option import *

def make_description():
    gcoption = ChoiceOption('name', 'GC name', ['ref', 'framework'], 'ref')
    gcdummy = BoolOption('dummy', 'dummy', default=False)
    objspaceoption = ChoiceOption('objspace', 'Object space',
                                ['std', 'thunk'], 'std')
    booloption = BoolOption('bool', 'Test boolean option', default=True)
    intoption = IntOption('int', 'Test int option', default=0)
    floatoption = FloatOption('float', 'Test float option', default=2.3)
    stroption = StrOption('str', 'Test string option', default="abc")
    boolop = BoolOption('boolop', 'Test boolean option op', default=True)
    wantref_option = BoolOption('wantref', 'Test requires', default=False)
    wantframework_option = BoolOption('wantframework', 'Test requires',
                                      default=False)
    
    gcgroup = OptionDescription('gc', '', [gcoption, gcdummy, floatoption])
    descr = OptionDescription('tiram', '', [gcgroup, booloption, objspaceoption,
                                           wantref_option, stroption,
                                           wantframework_option,
                                           intoption, boolop])
    return descr

def test_base_config():
    gcdummy = BoolOption('dummy', 'dummy', default=False)
    descr = OptionDescription('tiramisu', '', [gcdummy])
    cfg = Config(descr)
    assert cfg.dummy == False
    dm = cfg.unwrap_from_path('dummy')
    assert dm._name == 'dummy'
    
def test_base_config_and_groups():
    descr = make_description()
    # overrides the booloption default value
    config = Config(descr, bool=False)
    assert config.gc.name == 'ref'
    assert config.bool == False
    nm = config.unwrap_from_path('gc.name')
    assert nm._name == 'name'
    gc = config.unwrap_from_path('gc')
    assert gc._name == 'gc' 
    nm = config.unwrap_from_name('name')
    assert nm._name == 'name'
    
def test_base_config_in_a_tree():
    "how options are organized into a tree"
    descr = make_description()
    config = Config(descr, bool=False)
    
    assert config.gc.name == 'ref'
    config.gc.name = 'framework'
    assert config.gc.name == 'framework'
    assert getattr(config, "gc.name") == 'framework'

    assert config.objspace == 'std'
    config.objspace = 'thunk'
    assert config.objspace == 'thunk'
    
    assert config.gc.float == 2.3
    assert config.int == 0
    config.gc.float = 3.4
    config.int = 123
    assert config.gc.float == 3.4
    assert config.int == 123

    assert not config.wantref

    assert config.str == "abc"
    config.str = "def"
    assert config.str == "def"

    raises(AttributeError, 'config.gc.foo = "bar"')

    config = Config(descr, bool=False)
    assert config.gc.name == 'ref'
    config.wantframework = True

def test_config_values():
    "_cfgimpl_values appears to be a simple dict"
    descr = make_description()
    config = Config(descr, bool=False)
    config.set(dummy=False)
    assert config.gc._cfgimpl_values == {'dummy': False, 'float': 2.3, 'name': 'ref'}

def test_cfgimpl_get_home_by_path():
    descr = make_description()
    config = Config(descr, bool=False)
    assert config._cfgimpl_get_home_by_path('gc.dummy')[1] == 'dummy'
    assert config._cfgimpl_get_home_by_path('dummy')[1] == 'dummy'
    assert config.getpaths(include_groups=False) == ['gc.name', 'gc.dummy', 'gc.float', 'bool', 'objspace', 'wantref', 'str', 'wantframework', 'int', 'boolop']
    assert config.getpaths(include_groups=True) == ['gc', 'gc.name', 'gc.dummy', 'gc.float', 'bool', 'objspace', 'wantref', 'str', 'wantframework', 'int', 'boolop']   
